<?php

namespace Drupal\graphql_vertex_ai\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\key\KeyRepositoryInterface;
use Google\Cloud\DiscoveryEngine\V1\Client\CompletionServiceClient;
use Google\Cloud\DiscoveryEngine\V1\CompleteQueryRequest;
use Google\Cloud\DiscoveryEngine\V1\CompleteQueryResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The data producer for search queries.
 *
 * @DataProducer(
 *   id = "vertex_ai_autocomplete_query",
 *   name = @Translation("Search query"),
 *   description = @Translation("Search query"),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Search result")
 *   ),
 *   consumes = {
 *     "query" = @ContextDefinition("string",
 *       label = @Translation("Query"),
 *       required = TRUE
 *     ),
 *   }
 * )
 */
class VertexAiAutocompleteQuery extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\key\KeyRepositoryInterface $key_repository */
    $key_repository = $container->get('key.repository');
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $config_factory,
      $key_repository,
    );
  }

  /**
   * VertexAiSearchQuery constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The key repository service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    protected ConfigFactoryInterface $configFactory,
    protected KeyRepositoryInterface $keyRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * Resolver.
   *
   * @param string $query
   *   The search query.
   * @param \Drupal\graphql\GraphQL\Execution\FieldContext $metadata
   *   The metadata.
   *
   * @return \Google\Cloud\DiscoveryEngine\V1\CompleteQueryResponse
   *   The result.
   */
  public function resolve(string $query, FieldContext $metadata): CompleteQueryResponse {
    $config = $this->configFactory->get('graphql_vertex_ai.settings');
    $key = $this->keyRepository->getKey($config->get('key'))->getKeyValues();

    $autocompleteServiceClient = new CompletionServiceClient([
      'credentials' => $key,
    ]);

    $formattedDataStore = CompletionServiceClient::dataStoreName(
      $config->get('project_id'),
      $config->get('location'),
      $config->get('data_store_id'),
    );

    // Prepare the search request.
    $request = (new CompleteQueryRequest())
      ->setDataStore($formattedDataStore)
      ->setQuery($query);

    $response = $autocompleteServiceClient->completeQuery($request);

    $metadata->mergeCacheMaxAge(0);

    return $response;
  }

}
